from sqlparse.sql import Statement

from mag_db.bean.table_info import TableInfo
from mag_db.parser.base_parser import BaseParser


class DeleteParser(BaseParser):
    def _parse(self, stmt: Statement) -> TableInfo:
        # 提取表名及别名
        tables = super()._find_table(stmt, {'FROM'})

        # 提取条件部分
        where = super()._find_where(stmt)

        return TableInfo(tables, where=where)


from mag_tools.jsonparser.json_parser import JsonParser

from mag_db.handler.type_handler import T, TypeHandler


class DictTypeHandler(TypeHandler):
    def set_parameter(self, parameter: T) -> str | None:
        return JsonParser.from_bean(parameter) if isinstance(parameter, dict) else None


import re
from typing import Optional

from mag_tools.exception.dao_exception import DaoException
from mag_tools.log.logger import Logger
from mag_tools.enums.log_type import LogType
from mag_tools.utils.common.class_utils import ClassUtils
from sqlparse.sql import Function, Identifier, IdentifierList, Parenthesis, Statement, Token, Values, Where
from sqlparse.tokens import Keyword, String

from mag_db.bean.column import Column
from mag_db.bean.table import Table
from mag_db.bean.table_info import TableInfo


class BaseParser:
    def __init__(self, stmt: Statement):
        self.__sql = stmt.value

        try:
            self.__table_info = self._parse(stmt)
        except Exception as e:
            Logger.throw(LogType.DAO, f"SQL解析错误:{stmt.value}\n{e}")

    @property
    def sql(self):
        return self.__sql

    @property
    def table_info(self) -> TableInfo:
        return self.__table_info

    def _parse(self, stmt: Statement) -> TableInfo:
        raise NotImplementedError("_parse() must be implemented by subclasses")

    @classmethod
    def _find_keyword(cls, stmt: Statement, keyword: str) -> Optional[Keyword]:
        for token in stmt.tokens:
            if isinstance(token, Token) and token.ttype in Keyword and token.value.upper() == keyword.upper():
                return token
        return None

    @classmethod
    def _find_any_keyword(cls, stmt: Statement, keywords: set[str]) -> Optional[Keyword]:
        return next((cls._find_keyword(stmt, keyword) for keyword in keywords if cls._find_keyword(stmt, keyword)),
                    None)

    @classmethod
    def _get_value_of_token(cls, stmt: Statement, token: Token) -> Optional[str]:
        if not token:
            return None

        next_token = cls._next_token(stmt, token)
        while next_token and not ClassUtils.isinstance(next_token, (Identifier, IdentifierList, Parenthesis, Function)) and next_token.ttype != String.Single:
            next_token = cls._next_token(stmt, next_token)
        return next_token.value if next_token else None

    @classmethod
    def _find_values(cls, stmt: Statement) -> Optional[Values]:
        for token in stmt.tokens:
            if isinstance(token, Values):
                return token
        return None

    @classmethod
    def _find_where(cls, stmt: Statement) -> Optional[Where]:
        for token in stmt.tokens:
            if isinstance(token, Where):
                return token
        return None

    @classmethod
    def _find_parenthesis(cls, stmt: Statement):
        """
        查找并返回第一个括号的 Token。如果未找到，返回 None。
        """
        def find_in_tokens(tokens):
            for token in tokens:
                if isinstance(token, Parenthesis):
                    return token
                # 对嵌套类型进行递归检查
                if hasattr(token, 'tokens') and token.ttype is None:
                    nested_result = find_in_tokens(token.tokens)
                    if nested_result:
                        return nested_result
            return None

        return find_in_tokens(stmt.tokens)

    @classmethod
    def _next_token(cls, stmt: Statement, token: Token) -> Optional[Token]:
        if token is None:
            return None

        next_token_tuple = stmt.token_next(stmt.token_index(token))
        return next_token_tuple[1] if next_token_tuple and len(next_token_tuple) > 1 else None

    @classmethod
    def _find_table(cls, stmt: Statement, keywords: set[str]) -> list[Table]:
        # 提取表名及别名
        table_name_token = cls._find_any_keyword(stmt, keywords)
        table_full_name = cls._get_value_of_token(stmt, table_name_token)  # 包含列名
        table_name, table_alias = cls._parse_alias(table_full_name)

        if not table_name:
            raise DaoException(f'表名{table_name}不存在')

        return [Table(table_name, alias=table_alias)]

    @classmethod
    def _is_keyword(cls, str_: str) -> bool:
        keywords = ["INSERT", "UPDATE", "DELETE", "SELECT", "NOT", "LIKE", "WHERE", "FROM", "IN", "AS", "BY", "GROUP BY",
                    "ORDER BY", "COUNT", "ALIAS", "HAVING", "LIMIT", "OFFSET"]
        return str_ is not None and str_.upper() in keywords

    @classmethod
    def _parse_alias(cls, sql: str) -> Optional[tuple[str, Optional[str]]]:
        pattern = r'(\w+)(?:\s+AS\s+(\w+))?'
        match = re.search(pattern, sql, re.IGNORECASE)
        if match:
            name = match.group(1)
            alias = match.group(2) if match.group(2) else None
            return name, alias
        return None

    @classmethod
    def _parse_column_names(cls, column_names_str: str) -> list[Column]:
        columns = []
        items = column_names_str.split(',')
        for item in items:
            name, alias = cls._parse_alias(item)
            columns.append(Column(name, alias=alias))
        return columns


import os
import unittest

import pymysql
from mag_tools.config.sys_config import SysConfig

from mag_db.core.db_session import DBSession
from mag_db.manager.datasource_mgr import DatasourceMgr


class TestDBSession(unittest.TestCase):
    def setUp(self):
        SysConfig.set_root_dir(os.path.dirname(os.getcwd()))
        self.session = DatasourceMgr.create_session('Default')

    def test_connect_mysql(self):
        connection = self.session.connect()
        self.assertIsNotNone(connection)
        connection.close()

    def test_close(self):
        connection = self.session.connect()
        DBSession.close(connection)
        with self.assertRaises(pymysql.MySQLError):
            connection.ping(reconnect=False)

    def test_begin_transaction(self):
        trans = self.session.begin_transaction('test_tx')
        self.assertIsNotNone(trans)
        trans.commit()
        trans.close()

    def test_begin_new_transaction(self):
        trans = self.session.begin_new_transaction('test_tx')
        self.assertIsNotNone(trans)
        trans.commit()
        trans.close()

if __name__ == '__main__':
    unittest.main()




from mag_db.builder.base_sql import BaseSql
from mag_db.bean.table import Table
from mag_db.enums.operator_type import OperatorType
from mag_tools.enums.symbol import Symbol

class LoadData(BaseSql):

    def __init__(self, table_name: str, file: str, local: bool, field_terminated: str, enclosed: str, lines_terminated: str, ignore_lines: int):
        """
        构造LOAD DATA语句（只限于MySql）

        :param table_name: 表名
        :param file: 数据文件路径
        :param local: 文件是否从客户端本地加载
        :param field_terminated: 字段分隔符
        :param enclosed: 包含符
        :param lines_terminated: 行分隔符
        :param ignore_lines: 忽略的行数
        """
        super().__init__()

        self.append_operator(OperatorType.LOAD_DATA)
        if local:
            self.append_operator(OperatorType.LOCAL)
        self.append_operator(OperatorType.INFILE)
        self.append_string_with_symbol(file, Symbol.SINGLE_QUOTATION)

        self.append_operator(OperatorType.INTO_TABLE)
        self.append_tables([Table(table_name)])

        self.append_operator(OperatorType.FIELDS_TERMINATED).append_string_with_symbol(field_terminated, Symbol.SINGLE_QUOTATION)
        if enclosed:
            self.append_operator(OperatorType.ENCLOSED).append_string_with_symbol(enclosed, Symbol.SINGLE_QUOTATION)
        self.append_operator(OperatorType.LINES_TERMINATED).append_string_with_symbol(lines_terminated, Symbol.SINGLE_QUOTATION)
        self.append_operator(OperatorType.IGNORE).append_long(ignore_lines).append_operator(OperatorType.LINES)


import unittest
from datetime import date

from mag_db.data.index_parameters import IndexParameters


class TestBean:
    def __init__(self, name, age):
        self.name = name
        self.age = age

class TestIndexParameters(unittest.TestCase):
    def test_set_string(self):
        self.index_parameters = IndexParameters()
        self.index_parameters.set_value("test", str)
        self.assertEqual(self.index_parameters.parameters, (('test',),))
        self.assertEqual(len(self.index_parameters.type_handlers()), 1)

    def test_set_int(self):
        self.index_parameters = IndexParameters()
        self.index_parameters.set_value(123, int)
        self.assertEqual(self.index_parameters.parameters, ((123,),))
        self.assertEqual(len(self.index_parameters.type_handlers()), 1)

    def test_set_date(self):
        self.index_parameters = IndexParameters()
        test_date = date(2023, 1, 1)
        self.index_parameters.set_value(test_date, date)
        self.assertEqual(self.index_parameters.parameters, (test_date,))
        self.assertEqual(len(self.index_parameters.type_handlers()), 1)

    def test_set_bean(self):
        self.index_parameters = IndexParameters()
        bean = TestBean("Alice", 30)
        self.index_parameters.set_beans([bean], ["name", "age"])
        self.assertEqual((("Alice", 30),), self.index_parameters.parameters)
        self.assertEqual(2, len(self.index_parameters.type_handlers()))

    def test_set_field_map(self):
        self.index_parameters = IndexParameters()
        field_map = {"name": "Alice", "age": 30}
        self.index_parameters.set_field_maps([field_map], ["name", "age"])
        self.assertEqual(self.index_parameters.parameters, (("Alice", 30),))
        self.assertEqual(len(self.index_parameters.type_handlers()), 2)

if __name__ == '__main__':
    unittest.main()



from typing import Any

import pymysql
from mag_tools.exception.dao_exception import DaoException
from mag_tools.log.logger import Logger
from mag_tools.enums.log_type import LogType

from mag_db.core.transaction import Transaction
from mag_db.builder.select import Select
from mag_db.core.db_session import DBSession
from mag_db.manager.datasource_mgr import DatasourceMgr
from mag_db.sql.index_query_executor import IndexQueryExecutor
from mag_db.sql.index_delete_executor import IndexDeleteExecutor
from mag_db.sql.index_insert_executor import IndexInsertExecutor
from mag_db.sql.index_update_executor import IndexUpdateExecutor


class BaseDao:
    _session = None

    def __init__(self):
        datasource = DatasourceMgr.get_datasource('default')
        self._session = DBSession(datasource)

    def set_datasource(self, ds_name: str):
        self._session = DBSession(DatasourceMgr.get_datasource(ds_name))

    def insert_by_sql(self, insert_sql:str)->list[int or str]:
        insert = IndexInsertExecutor(insert_sql, self._session)
        return insert.execute()

    def update_by_sql(self, update_sql:str):
        update = IndexUpdateExecutor(update_sql, self._session)
        return update.execute()

    def query_by_sql(self, query_sql:str)->list[dict[str, Any]]:
        query = IndexQueryExecutor(query_sql, self._session)
        return query.execute_sql()

    def delete_by_sql(self, delete_sql:str):
        delete =IndexDeleteExecutor(delete_sql, self._session)
        return delete.execute()

    def total_count(self, table_name:str) -> int:
        select = Select([table_name], ['*'], None)
        query = IndexQueryExecutor(select.__str__(), self._session)

        return query.get_count()

    def begin_transaction(self, _name: str) -> Transaction:
        return self._session.begin_transaction(_name)

    def begin_new_transaction(self, _name: str) -> Transaction:
        return self._session.begin_new_transaction(_name)

    def test(self) -> bool:
        try:
            with self._session.connect() as conn:
                return conn is not None
        except (DaoException, pymysql.MySQLError) as e:
            Logger.error(LogType.DAO, e)
            return False

    @property
    def datasource_name(self):
        return self._session.datasource_name


from typing import Any

from mag_db.handler.type_handler import T, TypeHandler


class StringTypeHandler(TypeHandler):
    def set_parameter(self, parameter: T) -> Any | None:
        return parameter


from dataclasses import dataclass, field
from typing import Any, Optional

from mag_db.bean.foreign_key import ForeignKey
from mag_db.bean.primary_key import PrimaryKey
from mag_db.bean.column import Column
from mag_db.bean.table import Table

@dataclass
class TableInfo:
    tables: list[Table] = field(default_factory=list, metadata={'description': '表信息列表'})
    columns: list[Column] = field(default_factory=list, metadata={'description': '列信息列表'})
    primary_key: Optional[PrimaryKey] = field(default=None, metadata={'description': '主键'})
    foreign_keys: list[ForeignKey] = field(default_factory=list, metadata={'description': '外键列表'})
    distinct: bool = field(default=False, metadata={'description': '去除重复的数据行'})
    values: Optional[list[Any]] = field(default_factory=list, metadata={'description': '数据集'})
    where: Optional[str] = field(default=None, metadata={'description': '条件语句'})
    group_by:Optional[str] = field(default=None, metadata={'description': '分组语句'})
    having: Optional[str] = field(default=None, metadata={'description': '过滤语句'})
    order_by: Optional[str] = field(default=None, metadata={'description': '排序语句'})
    charset: Optional[str] = field(default=None, metadata={'description': '字符集'})
    engine: Optional[str] = field(default=None, metadata={'description': '引擎'})
    comment: Optional[str] = field(default=None, metadata={'description': '备注'})
    column_name_map: Optional[dict[str, str]] = field(default=None, metadata={'description': '列名映射表'})
    auto_increment: bool = field(default=False, metadata={'description': '是否自动递增'})

    def __post_init__(self):
        if not self.column_name_map:
            self.column_name_map = TableInfo.__get_column_name_map(self.columns)

    @property
    def table_names(self) -> list[str]:
        return [table.name for table in self.tables]

    def get_column_names(self, is_query: bool = False) -> list[str]:
        column_names = [column.name for column in self.columns]

        # 对于非查询类操作，如主键是自增型，则排除
        if not is_query and self.auto_increment and self.primary_key:
            column_names.remove(self.primary_key.default_column)

        return column_names

    @property
    def default_primary_key(self) -> Optional[str]:
        return self.primary_key.default_column if self.primary_key else None

    def __str__(self):
        foreign_keys_str_list = [foreign_key.__str__() for foreign_key in self.foreign_keys]

        parts = [ f"tables=[{', '.join(self.table_names)}]",
                  f"columns=[{', '.join(self.get_column_names())}]" if self.columns else "",
                  f"primary_key={self.primary_key}" if self.primary_key else "",
                  f"foreign_keys=[{', '.join(foreign_keys_str_list)}]" if len(self.foreign_keys) > 0 else "",
                  f"distinct={self.distinct}",
                  f"values={self.values}" if self.values else "",
                  f"where={self.where}" if self.where else "",
                  f"group_by={self.group_by}" if self.group_by else "",
                  f"having={self.having}" if self.having else "",
                  f"order_by={self.order_by}" if self.order_by else "",
                  f"column_name_map={self.column_name_map}" if self.column_name_map else ""
        ]
        # 过滤掉空字符串
        parts = [part for part in parts if part]
        return f"TableInfo({', '.join(parts)})"

    @classmethod
    def __get_column_name_map(cls, columns: list[Column]) -> dict[str, str]:
        mappings = {}
        for column in columns:
            map_value = column.mapping
            if map_value:
                mappings[column.name] = map_value
        return mappings




from mag_db.bean.table import Table
from mag_db.builder.where import Where
from mag_db.builder.base_sql import BaseSql
from mag_db.enums.operator_type import OperatorType


class Delete(BaseSql):
    """
    DELETE语句操作类

    @author: Xiaolong Cao
    @version: v1.0
    @description: 用于构建DELETE语句。
    @date: 2019/8/30
    """
    def __init__(self, table_names: list[str], where: Where = None):
        super().__init__()

        self.append_operator(OperatorType.DELETE) \
            .append_tables([Table(name) for name in table_names])

        if where is not None:
            self.append_condition(where.__str__())

if __name__ == "__main__":
    # 示例代码：使用Delete类
    _table_names = ["tableName1", "tableName2"]
    _where = Where("columnName = ?")
    delete_sql = Delete(_table_names, _where)
    print(delete_sql)



import json
from dataclasses import dataclass
from typing import Any, Optional

from pydantic import BaseModel, Field

from mag_db.enums.db_type import DbType


@dataclass
class Datasource(BaseModel):
    name: Optional[str] = Field(None, alias='数据源名')
    sn: Optional[int] = Field(None, description="数据源序号")
    host: Optional[str] = Field(None, alias="host")
    port: Optional[int] = Field(None, alias="port")
    username: Optional[str] = Field(None, alias="username")
    password: Optional[str] = Field(None, alias="password")
    db_name: Optional[str] = Field(None, alias="db_name")
    db_type: Optional[str] = Field(DbType.MYSQL, alias="db_type")
    driver_class_name: Optional[str] = Field(None, alias="driver-class-name")
    max_wait: Optional[int] = Field(3000, alias="max-wait")
    min_idle: Optional[int] = Field(5, alias="min-idle")
    max_idle: Optional[int] = Field(50, alias="max-idle")
    max_active: Optional[int] = Field(50, alias="max-active")
    test_on_borrow: Optional[bool] = Field(True, alias="test-on-borrow")
    validate_query: Optional[str] = Field("SELECT 1", alias="validate-query")
    initial_size: Optional[int] = Field(None, alias="initial-size")
    time_between_evict: Optional[int] = Field(None, alias="time-between-eviction-runs-millis")
    num_tests_per_evict: Optional[int] = Field(None, alias="num-tests-per-eviction-run")
    min_evict_idle_time: Optional[int] = Field(None, alias="min-evictable-idle-time-millis")
    max_insert: Optional[int] = Field(1000, alias="max-insert")
    fields_terminated: Optional[str] = Field("\\t", alias="fields-terminated")
    enclosed: Optional[str] = Field(None, alias="enclosed")
    lines_terminated: Optional[str] = Field("\\r\\n", alias="lines-terminated")
    ignore_lines: Optional[int] = Field(1, alias="ignore-lines")
    encrypted: Optional[bool] = Field(False, description="是否加密密码")
    crypt_alg: Optional[str] = Field("AES", description="加密算法，缺省为AES")
    encrypt_key: Optional[str] = Field(None, description="加密密钥")
    encoding: Optional[str] = Field("utf-8", description="编码格式")
    default_page_size: Optional[int] = Field(10, description="查询时缺省页面大小")
    log_sql: Optional[bool] = Field(True, description="是否记录SQL语句")
    evict: Optional[bool] = Field(True, description="连接池后台是否清理")
    server_timezone: Optional[str] = Field("GMT+8", description="服务器时区")
    use_ssl: Optional[bool] = Field(False, description="是否使用SSL")
    auto_reconnect: Optional[bool] = Field(True, description="是否自动重连接")
    fail_over_read_only: Optional[bool] = Field(False, description="是否只读失败转移")
    allow_public_key_retrieval: Optional[bool] = Field(True, description="是否允许公钥检索")
    comment: Optional[str] = Field(None, description="备注")
    table_number: Optional[int] = Field(None, description="表数目")

    def __init__(self, **data):
        super().__init__(**data)

    @property
    def can_connect(self) -> bool:
        """
        判定是否可直联
        :return: 是否可直联
        """
        try:
            self.check()
            return self.host is not None and self.username is not None and self.username != "" and self.db_name is not None and self.db_name != ""
        except Exception as e:
            print(f"Error: {e}")
            return False

    def check(self):
        """
        检查数据源信息的有效性
        """
        if not self.url:
            raise ValueError("URL不能为空")
        if not self.username:
            raise ValueError("用户名不能为空")
        if not self.password:
            raise ValueError("密码不能为空")

    @classmethod
    def from_config(cls, config: dict[str, Any]):
        """
        从配置信息中加载字段
        :param config: 配置信息
        """
        try:
            ds_info = config.get("datasource") if "datasource" in config else config
            return cls(**ds_info)
        except Exception as e:
            print(f"Error loading configuration from dict: {e}")

    @classmethod
    def from_file(cls, cfg_file):
        """
        从文件中读取配置并加载字段
        :param cfg_file: 配置文件路径
        """
        try:
            with open(cfg_file, 'r', encoding='utf-8') as file:
                data = json.load(file)
                ds_info = data.get("datasource") if "datasource" in data else data.get("datasource")

            return cls.from_config(ds_info)
        except Exception as e:
            print(f"Error loading configuration from file: {e}")


if __name__ == "__main__":
    _ds = Datasource.from_file('..\\..\\docs\\config\\db_config.json')
    print(_ds)



from typing import Optional

from mag_tools.enums.symbol import Symbol

from mag_db.bean.column import Column
from mag_db.bean.table import Table
from mag_db.builder.sql_builder import SqlBuilder
from mag_db.builder.where import Where
from mag_db.enums.operator_type import OperatorType
from mag_db.enums.relation import Relation
from mag_db.enums.sql_type import SqlType
from mag_db.parser.sql_parser import SqlParser


class BaseSql:
    def __init__(self, sql: Optional[str] = None):
        self.builder = SqlBuilder()
        self.tables = SqlParser(sql).tables

    def append_operator(self, operator_type:OperatorType):
        self.builder.append_operator(operator_type)
        return self

    def append_symbol(self, symbol:Symbol):
        self.builder.append_symbol(symbol)
        return self

    def append_string(self, string:str):
        self.builder.append_string(string)
        return self

    def append_string_with_symbol(self, string: str, symbol: Symbol):
        self.builder.append_string_with_symbol(string, symbol)
        return self

    def append_long(self, long: int):
        self.builder.append_long(long)
        return self

    def append_long_with_paren(self, long: int):
        self.builder.append_long_with_paren(long)
        return self

    def remove_last_char(self):
        self.builder.remove_last_char()
        return self

    def remove_last_keyword(self, keyword: str):
        self.builder.remove_last_keyword(keyword)
        return self

    def append_relation(self, relation: Relation):
        self.builder.append_relation(relation)
        return self

    def append_with_paren(self, columns: list[Column], is_multiple_table: bool):
        self.builder.append_with_paren(columns, is_multiple_table)
        return self

    def append_comment(self, comment: str):
        self.builder.append_string_with_symbol(comment, Symbol.SINGLE_QUOTATION)
        return self

    def clear(self):
        self.builder.clear()

    def get_param_count(self):
        return self.builder.__str__().count(Symbol.PLACE_HOLDER.code)

    def __str__(self):
        return self.builder.__str__()

    def append_tables(self, tables: list[Table]):
        self.tables = tables

        for i, table in enumerate(tables):
            if i > 0:
                self.builder.append_symbol(Symbol.COMMA)
            self.builder.append_table(table)

        return self

    def append_column(self, column: Column, has_table_name: bool, has_alias: bool):
        self.builder.append_column(column, has_table_name, has_alias)
        return self

    def append_whole_columns(self, columns: list[Column]):
        for column in columns:
            self.append_whole_column(column)

    def append_simple_columns(self, columns: list[Column]):
        for column in columns:
            self.append_column(column, False, False).append_symbol(Symbol.COMMA)

    def append_condition(self, where: Where):
        if where:
            self.builder.append_string(str(where))
        return self

    def replace(self, old_chars: str, new_chars: str):
        self.builder = SqlBuilder(self.builder.__str__().replace(old_chars, new_chars))

    @classmethod
    def to_columns(cls, column_names: list[str]) -> list[Column]:
        return [Column(name[:-1] if name.endswith('_') else name) for name in column_names]

    def append_whole_column(self, column: Column):
        self.append_string(column.name)

        sql_type = column.sql_type
        if sql_type:
            self.append_string(sql_type.code)

            if sql_type in [SqlType.CHAR, SqlType.VARCHAR]:
                column_size = column.column_size or 255

                if column.masking_alg and (column.masking_alg.is_hash() or column.masking_alg.is_crypt()):
                    if sql_type == SqlType.CHAR:
                        if column_size < 255 - 32:
                            column_size = 255
                        else:
                            column_size += 128
                            sql_type = SqlType.VARCHAR
                    else:
                        column_size += 128

                self.append_long_with_paren(column_size)
            elif sql_type == SqlType.BIT:
                column_size = column.column_size or 64
                self.append_long_with_paren(column_size)
            elif sql_type == SqlType.VARBINARY:
                column_size = column.column_size or 1024
                self.append_long_with_paren(column_size)
            elif sql_type in [SqlType.DATETIME, SqlType.TIMESTAMP]:
                decimal_digits = column.decimal_digits
                if decimal_digits and decimal_digits > 0:
                    self.append_long_with_paren(decimal_digits)
            elif sql_type in [SqlType.TINYINT, SqlType.INT]:
                if column.is_zero_filling:
                    self.append_long_with_paren(10)
                if column.is_unsigned:
                    self.append_operator(OperatorType.UNSIGNED)
            elif sql_type in [SqlType.DECIMAL, SqlType.DOUBLE, SqlType.FLOAT]:
                if column.column_size and column.decimal_digits:
                    self.builder.append_symbol(Symbol.OPEN_PAREN)
                    self.append_long(column.column_size)
                    self.builder.append_symbol(Symbol.COMMA)
                    self.append_long(column.decimal_digits)
                    self.builder.append_symbol(Symbol.CLOSE_PAREN)
            elif sql_type == SqlType.ENUM:
                self.builder.append_symbol(Symbol.OPEN_PAREN)
                self.append_string_with_symbol(column.enum_values, Symbol.SINGLE_QUOTATION)
                self.builder.append_symbol(Symbol.CLOSE_PAREN)

        self.append_string(OperatorType.NULL.code if column.is_can_null else OperatorType.NOT_NULL.code)

        if column.is_auto_increment:
            self.append_string(OperatorType.AUTO_INCREMENT.name)

        default_value = column.default_value
        if default_value:
            self.append_operator(OperatorType.DEFAULT)

            if "CURRENT_TIMESTAMP" in default_value.upper():
                if not column.decimal_digits:
                    idx = default_value.find("(")
                    if idx > 0:
                        default_value = default_value[:idx]
                self.append_string(default_value)
            elif sql_type == SqlType.BIT:
                default_value = default_value.replace("b", "").replace("'", "")
                self.append_string(default_value)
            else:
                self.append_string_with_symbol(default_value, Symbol.SINGLE_QUOTATION)
        else:
            if not column.is_primary_key and column.is_can_null:
                self.append_string(OperatorType.DEFAULT.code).append_operator(OperatorType.NULL)

        if column.comment:
            self.append_operator(OperatorType.COMMENT).append_comment(column.comment)
        self.builder.append_symbol(Symbol.COMMA)



from enum import Enum
from typing import get_args, TypeVar, Any

from mag_tools.enums.base_enum import BaseEnum
from mag_tools.utils.common.class_utils import ClassUtils
from mag_tools.jsonparser.json_parser import JsonParser

from mag_db.data.db_output import DbOutput

V = TypeVar('V')

class ResultSetHelper:
    @staticmethod
    def to_maps(result_set: list[dict[str, V]], db_output: DbOutput) -> list[dict[str, V]]:
        return [
            {
                db_output.get_target_name(column_name): row_map.get(column_name, None) for column_name in db_output.column_names
            }
            for row_map in result_set
        ]

    @staticmethod
    def to_beans(result_set: list[dict[str, V]], db_output: DbOutput) -> list[V]:
        result_list = []
        for row_map in result_set:
            kwargs = {}
            for column_name in db_output.column_names:
                target_name = db_output.get_target_name(column_name)
                target_type = ClassUtils.get_field_type_of_class(db_output.result_class, target_name)
                target_type = ClassUtils.get_origin_type(target_type)
                value = row_map.get(column_name)

                if target_type is tuple:
                    args = get_args(target_type)
                    if args:
                        kwargs[target_name] = JsonParser.to_tuple(value, args[0])
                elif target_type is list:
                    args = get_args(target_type)
                    if args:
                        kwargs[target_name] = JsonParser.to_list(value, args[0])
                elif target_type is dict:
                    args = get_args(target_type)
                    if args:
                        kwargs[target_name] = JsonParser.to_map(value, str, args[0])
                elif issubclass(target_type, Enum):
                    kwargs[target_name] = BaseEnum.of(target_type, value)
                else:
                    kwargs[target_name] = value

            result_list.append(db_output.result_class(**kwargs))

        return result_list



from typing import Optional

from pymysql.connections import Connection

from mag_tools.exception.dao_exception import DaoException
from mag_tools.log.logger import Logger
from mag_tools.enums.log_type import LogType

from mag_db.core.transaction_cache import TransactionCache


class Transaction:
    def __init__(self, name: str, datasource_name:str, connection: Optional[Connection]=None):
        self.is_commit = False
        self.is_rollback = False
        self.is_end = False
        self.name = name
        self.__datasource_name = datasource_name
        self.__connection = connection
        self.parent = None

    @classmethod
    def from_parent(cls, parent, name: str):
        if parent is None:
            raise DaoException("parent transaction is null")

        parent.assert_open()
        return cls(name, parent.datasource_name, parent.get_connection())

    @property
    def datasource_name(self):
        return self.parent.datasource_name if self.parent else self.__datasource_name

    @property
    def connection(self)->Connection:
        return self.parent.connection() if self.parent else self.__connection

    def begin_with_connection(self):
        try:
            if self.connection is None:
                raise DaoException("connection is null")

            self.connection.autocommit(False)
            TransactionCache.append_tx(self.datasource_name, self)
        except Exception as e:
            self.close()
            Logger.throw(LogType.DAO, f"启动事务[{self.name}]失败: {str(e)}")

    def begin_without_connection(self):
        TransactionCache.append_tx(self.datasource_name, self)

    def commit(self):
        if self.is_commit or self.is_end:
            Logger.throw(LogType.DAO, f"提交事务[{self.name}]失败")

        try:
            self.do_commit()
        finally:
            self.is_commit = True

    def rollback(self):
        if self.is_rollback or self.is_end:
            Logger.throw(LogType.DAO, f"回滚事务[{self.name}]失败")

        try:
            self.do_rollback()
        finally:
            self.is_rollback = True

    def end(self):
        if self.is_end:
            Logger.throw(LogType.DAO, f"结束事务[{self.name}]失败")

        try:
            self.do_end()
            if not self.is_commit and not self.is_rollback:
                Logger.throw(LogType.DAO, f"结束事务[{self.name}]失败")
        finally:
            self.is_end = True

    def assert_open(self):
        if self.is_end or self.is_commit or self.is_rollback:
            Logger.throw(LogType.DAO,  f"事务[{self.name}]未打开")

    def do_rollback(self):
        try:
            if self.parent is None and self.connection:
                self.connection.rollback()
                self.connection.autocommit(True)
        except Exception as e:
            Logger.throw(LogType.DAO, f"回滚事务[{self.name}]失败：{str(e)}")

    def do_end(self):
        try:
            if self.parent is None:
                self.close()
        finally:
            TransactionCache.remove_tx(self.datasource_name, self)

    def close(self):
        try:
            if self.__connection:
                self.__connection.close()
        except Exception as e:
            Logger.throw(LogType.DAO, f"关闭事务[{self.name}]失败：{str(e)}")
        finally:
            self.__connection = None

    def do_commit(self):
        try:
            if self.parent is None:
                self.__connection.commit()
                self.__connection.autocommit(True)
        except Exception as e:
            Logger.throw(LogType.DAO, f"提交事务[{self.name}]失败：{str(e)}")


from mag_tools.enums.base_enum import BaseEnum


class Relation(BaseEnum):
    """
    关系比较类型的枚举类

    作者: xlcao
    版本: 1.0
    版权所有: Copyright (c) 2016 by Xiaolong Cao. All rights reserved.
    描述: MORE_THAN 大于；LESS_THAN 小于；EQUAL 等于。
    日期: 2019/8/12
    """
    MORE_THAN = (" > ", "大于")
    MORE_THAN_EQUAL = (" >= ", "大于等于")
    LESS_THAN = (" < ", "小于")
    LESS_THAN_EQUAL = (" <= ", "小于等于")
    EQUAL = (" = ", "等于")
    NOT_EQUAL = (" != ", "不等于")
    LIKE = (" LIKE ", "类似")
    NOT_LIKE = (" NOT LIKE ", "不类似")
    IN = (" IN ", "包含")
    NOT_IN = (" NOT IN ", "不包含")


    @classmethod
    def of(cls, code: str):
        code = f" {code.strip()} "
        for item in cls:
            if item.code == code:
                return item
        raise ValueError(f"{code} is not a valid {cls.__name__}")



from unittest import TestCase

import sqlparse

from mag_db.parser.create_parser import CreateParser
from mag_db.parser.delete_parser import DeleteParser
from mag_db.parser.drop_parser import DropParser
from mag_db.parser.insert_parser import InsertParser
from mag_db.parser.query_parser import QueryParser
from mag_db.parser.update_parser import UpdateParser


class TestAllParser(TestCase):
    def test_create_parser(self):
        _sql = '''
        CREATE TABLE IF NOT EXISTS table_name1 (
            id INT  COMMENT '主键ID', 
            name VARCHAR(50) NOT NULL COMMENT '名称',
            length1 DECIMAL(10,2) NULL COMMENT '长度,缺省值为0',
            created_at TIMESTAMP(6) DEFAULT CURRENT_TIMESTAMP COMMENT "创建时间,缺省为当天", 
            FOREIGN KEY (id,name) REFERENCES other_table(id,name),
            CONSTRAINT pk_name PRIMARY KEY (id, name) 
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='示例表';
        '''

        _stmt = sqlparse.parse(_sql)[0]
        parser = CreateParser(_stmt)
        print(f'SQL: {parser.sql}')

        if parser.table_info:
            print(parser.table_info)

    def test_drop_parser(self):
        _sql = "DROP TABLE IF EXISTS table_name1;"

        _stmt = sqlparse.parse(_sql)[0]
        parser = DropParser(_stmt)
        if parser.table_info:
            print(parser.table_info)

    def test_insert_parser(self):
        _sql = "INSERT INTO table_name1 (column1, column2) VALUES ('value1', 'value2');"

        _stmt = sqlparse.parse(_sql)[0]
        _parser = InsertParser(_stmt)
        if _parser.table_info:
            print(_parser.table_info)

    def test_delete_parser(self):
        _sql = "DELETE FROM table_name1 AS t WHERE id = 1;"

        _stmt = sqlparse.parse(_sql)[0]

        parser = DeleteParser(_stmt)
        if parser.table_info:
            print(parser.table_info)

    def test_query_parser(self):
        _sql = "SELECT t.column1 AS c1, t.column2 AS c2 FROM table_name1 AS t WHERE t.id IN (1, 2, 3) AND t.name = 'John' GROUP BY t.column1 HAVING COUNT(t.column1) > 1 ORDER BY t.column2;"
        _stmt = sqlparse.parse(_sql)[0]
        _parser = QueryParser(_stmt)
        if _parser.table_info:
            print(_parser.table_info)

    def test_update_parser(self):
        _sql = "UPDATE table_name1 AS t SET t.column1 = 'value1', t.column2 = 'value2' WHERE id = 1;"

        _stmt = sqlparse.parse(_sql)[0]
        _result = UpdateParser(_stmt)
        if _result.table_info:
            print(_result.table_info)



from mag_tools.enums.symbol import Symbol

from mag_db.bean.column import Column
from mag_db.bean.table import Table
from mag_db.builder.base_sql import BaseSql
from mag_db.enums.operator_type import OperatorType
from mag_db.enums.relation import Relation


class Insert(BaseSql):
    """
    INSERT语句操作类

    @description: 用于构建INSERT语句。
    @version: v1.0
    @date: 2019/8/30
    """
    def __init__(self, table_names: list[str], column_names: list[str], record_count: int, duplicate_update: bool = False):
        """
        构造方法

        :param table_names: the database table name
        :param column_names: the list of column names which need to be inserted
        :param record_count: the count of records
        :param duplicate_update: 是否更新重复记录
        """
        super().__init__()

        self.append_operator(OperatorType.INSERT).append_tables([Table(name) for name in table_names])
        self.__append_columns(column_names, record_count)

        if duplicate_update:
            self.append_operator(OperatorType.DUPLICATE_UPDATE)

            is_first = True
            for column_name in column_names:
                if is_first:
                    is_first = False
                else:
                    self.append_symbol(Symbol.COMMA)

                self.append_column(Column(column_name), True, False).append_relation(Relation.EQUAL).append_operator(OperatorType.VALUES).append_symbol(Symbol.OPEN_PAREN).append_column(Column(column_name), True, False).append_symbol(Symbol.CLOSE_PAREN)

    def __append_columns(self, column_names: list[str], record_count: int) -> 'Insert':
        is_multi_table = len(self.tables) > 1

        self.append_with_paren(self.to_columns(column_names), is_multi_table).append_operator(OperatorType.VALUES)
        self.__append_values(len(column_names), record_count)

        return self

    def __append_values(self, param_count: int, record_count: int):
        """
        return the value part of a sql prepareStatement

        :param param_count: the size of question mark, aka ?
        :param record_count: the size of records
        """
        for i in range(record_count):
            if i > 0:
                self.append_symbol(Symbol.COMMA)
            self.append_symbol(Symbol.OPEN_PAREN)

            for j in range(param_count - 1):
                self.append_symbol(Symbol.PLACE_HOLDER)
                self.append_symbol(Symbol.COMMA)

            if param_count > 0:
                self.append_symbol(Symbol.PLACE_HOLDER)

            self.append_symbol(Symbol.CLOSE_PAREN)


from mag_tools.exception.dao_exception import DaoException
from mag_tools.utils.common.string_utils import StringUtils
from sqlparse.sql import Statement

from mag_db.bean.table import Table
from mag_db.bean.table_info import TableInfo
from mag_db.parser.base_parser import BaseParser


class InsertParser(BaseParser):
    def _parse(self, stmt: Statement) -> TableInfo:
        # 提取表名/表别名、列名/列别名
        table_name_token = super()._find_keyword(stmt, 'INTO')
        table_full_name = super()._get_value_of_token(stmt, table_name_token)  # 包含列名
        table_name, column_names_str, _ = StringUtils.split_by_keyword(table_full_name, '()')

        # 提取表名
        if not table_name:
            raise DaoException(f'表名不存在:{stmt.value}')

        tables = [Table(table_name)]

        # 提取列名及其别名
        columns = super()._parse_column_names(column_names_str)

        # 提取值
        values_token = super()._find_values(stmt)
        values = [token.value for token in values_token.tokens if token.ttype is None]

        return TableInfo(tables, columns, values=values)


from dataclasses import dataclass, field


@dataclass
class ForeignKey:
    """
    描述一个外键约束
    """
    column_names: list[str] = field(default_factory=list, metadata={'description': '外键列名列表'})
    referenced_table: str = field(default=None, metadata={'description': '引用的表名'})
    referenced_columns: list[str] = field(default_factory=list, metadata={'description': '引用的列名列表'})

    def __str__(self):
        """
        返回外键约束的字符串表示

        返回:
        str: 外键约束的字符串表示
        """
        column_names_str = ', '.join(self.column_names)
        referenced_columns_str = ', '.join(self.referenced_columns)
        return f"FOREIGN KEY ({column_names_str}) REFERENCES {self.referenced_table}({referenced_columns_str})"


from mag_tools.enums.base_enum import BaseEnum


class SynchronMode(BaseEnum):
    """
    数据同步方式的枚举类
    大数据平台的同步方式有几种：
    全量同步：每天新增一个日期分区，同步并存储当天的全量数据，适用于小数据情况；
    全量覆盖：不设置分区，每天直接覆盖原数据；
    实时增量同步：不设置分区，定时上传新增数据。
    """
    FULL_COVER = ("full_coverage", "全量覆盖")
    REAL_INCREASE = ("realtime_increase", "实时增量同步")
    FULL_SYNC = ("full_synchron", "全量同步")



from datetime import date, time
from typing import Any, Optional

import pymysql
from mag_tools.exception.dao_exception import DaoException
from mag_tools.log.logger import Logger
from mag_tools.enums.log_type import LogType
from mag_tools.enums.symbol import Symbol
from pymysql import Connection
from pymysql.cursors import Cursor
from win32ctypes.pywin32.pywintypes import datetime

from mag_db.core.db_session import DBSession
from mag_db.core.transaction import Transaction
from mag_db.data.index_parameters import IndexParameters, K, T, V


class BaseExecutor:
    def __init__(self, sql: str, session: DBSession):
        if not sql:
            raise ValueError('sql cannot be empty')

        self._sql = sql
        self._session = session
        self._indexParameters = IndexParameters()

    def get_connection(self, tx: Optional[Transaction] = None)->Connection:
        con = None

        try:
            # 取得当前未结束的事务
            if tx:
                tx.assert_open()
                # 如果SQL操作属于某个事务，则从该事务中取得连接
                con = tx.connection
            else:
                # 无事务则直接取得新的连接
                if self._session is None:
                    raise DaoException("找不到有用数据库会话")

                con = self._session.connect()

            if con is None:
                raise DaoException("数据库连接不能为空")
        except DaoException as e:
            Logger.throw(LogType.DAO, f"取数据库连接失败: {str(e)}")

        return con

    @classmethod
    def close_connection(cls, con: Connection, tx: Transaction = None, cursor: Cursor = None):
        try:
            if cursor:
                cursor.close()
        except pymysql.MySQLError:
            pass

        try:
            # 无事务则直接关闭连接, 否则数据库连接随相应事务关闭而关闭
            if tx is None and con:
                con.close()
        except pymysql.MySQLError:
            pass

        try:
            if tx:
                tx.close()
        except pymysql.MySQLError:
            pass

    def prepare(self)->Optional[tuple[tuple[Any,...], ...]]:
        if self._indexParameters and not self._indexParameters.is_empty():
            return self._indexParameters.get_values()
        return None

    def check(self) -> None:
        num_of_holder = self._sql.count(Symbol.PLACE_HOLDER.code)
        num_of_row = self._indexParameters.row_count
        num_of_field = self._indexParameters.field_num

        if num_of_holder * num_of_row != self._indexParameters.sum_of_params+ num_of_field:
            msg = (f"参数个数不匹配(总参数={self._indexParameters.sum_of_params}，占位符={num_of_holder}, 行数={num_of_row}，条件字段={num_of_field})\n"
                   f"参数为：{self._indexParameters.parameters}")
            Logger.throw(LogType.DAO, msg)

    def clear_parameters(self) -> None:
        self._indexParameters.clear()

    def set_string(self, parameter: str) -> None:
        self._indexParameters.set_value(parameter, str)

    def set_bytes(self, parameter: bytes) -> None:
        self._indexParameters.set_value(parameter, bytes)

    def set_date(self, parameter: date) -> None:
        self._indexParameters.set_value(parameter, date)

    def set_time(self, parameter: time) -> None:
        self._indexParameters.set_value(parameter, time)

    def set_datetime(self, parameter: datetime) -> None:
        self._indexParameters.set_value(parameter, datetime)

    def set_bool(self, parameter: bool) -> None:
        self._indexParameters.set_value(parameter, bool)

    def set_beans(self, parameter: list[T], column_names, column_name_map: dict[str, str]) -> None:
        self._indexParameters.set_beans(parameter, column_names, column_name_map)

    def set_maps(self, parameter: list[dict[K, V]], column_names) -> None:
        self._indexParameters.set_field_maps(parameter, column_names)

    def set_fields_of_where(self, fields: list) -> None:
        self._indexParameters.set_fields_of_where(fields)


from typing import Any, Optional, Type, TypeVar

import pymysql
from mag_tools.bean.db_list import DbList
from mag_tools.bean.db_page import DbPage
from mag_tools.exception.dao_exception import DaoException
from mag_tools.log.logger import Logger
from mag_tools.enums.log_type import LogType

from mag_db.core.db_session import DBSession
from mag_db.data.db_output import DbOutput
from mag_db.data.result_set_helper import ResultSetHelper
from mag_db.sql.base_executor import BaseExecutor
from mag_db.utils.dao_utils import DaoUtils

T = TypeVar('T')
class IndexQueryExecutor(BaseExecutor):
    def __init__(self, sql: str, session: DBSession):
        super().__init__(sql, session)

    def execute_sql(self)->list[dict[str, Any]]:
        return self.__fetchall()

    def execute_bean(self, bean_class: Type, column_names: list[str], column_name_map: dict[str, str] = None) -> T:
        beans = self.execute_beans(bean_class, column_names, column_name_map)
        return beans[0] if beans else None

    def execute_beans(self, bean_class: Type, column_names: list[str], column_name_map: dict[str, str] = None, is_multi_table: bool = False) -> DbList[T]:
        result_set = self.__fetchall()

        output = DbOutput.from_class(bean_class, column_names, column_name_map)
        output.is_multi_table = is_multi_table
        data = ResultSetHelper.to_beans(result_set, output)
        return DbList(data)

    def execute_beans_by_page(self, bean_class: Type, column_names: list[str], column_name_map: Optional[dict[str, str]] = None,
                              page: Optional[DbPage] = None, is_multi_table: bool = False) -> DbList[T]:
        result_set = self.__fetchall(page)
        total_count = self.get_count()

        output = DbOutput.from_class(bean_class, column_names, column_name_map)
        output.is_multi_table = is_multi_table

        data = ResultSetHelper.to_beans(result_set, output)
        return DbList(data, page, total_count)

    def execute_map(self, column_names: list[str]) -> dict[str, Any]:
        maps = self.execute_maps(column_names)
        return maps[0] if maps else {}

    def execute_maps(self, column_names: list[str]) -> DbList[dict[str, Any]]:
        output = DbOutput.from_class(None, column_names)
        result_set = self.__fetchall()
        data = ResultSetHelper.to_maps(result_set, output)
        return DbList(data)

    def execute_maps_by_page(self, column_names: list[str], page: Optional[DbPage] = None) -> DbList:
        result_set = self.__fetchall(page)
        total_count = self.get_count()

        output = DbOutput.from_class(dict, column_names)
        data = ResultSetHelper.to_maps(result_set, output)
        return DbList(data, page, total_count)

    def get_count(self) -> int:
        try:
            count_sql = DaoUtils.get_count_sql(self._sql)

            query_executor = IndexQueryExecutor(count_sql, self._session)
            record = query_executor.execute_map(['COUNT_'])
            return record.get("COUNT_", 0)
        except DaoException as dao:
            Logger.info(LogType.DAO, dao)
            return 0

    def __fetchall(self, page: Optional[DbPage] = None) -> list[dict[str, Any]]:
        self.check()

        try:
            values = self.prepare()

            with self.get_connection() as conn:
                with conn.cursor() as cursor:
                    sql = f"{self._sql} {page.get_sql()}" if page else self._sql
                    if values:
                        cursor.execute(sql, values)
                    else:
                        cursor.execute(sql)

                    column_names = [description[0] for description in cursor.description]
                    return [dict(zip(column_names, row)) for row in cursor.fetchall()]
        except pymysql.MySQLError:
            Logger.throw(LogType.DAO, f"查询时出错: {self._sql}")


from sqlparse.sql import Statement

from mag_db.bean.table_info import TableInfo
from mag_db.parser.base_parser import BaseParser


class DropParser(BaseParser):
    def _parse(self, stmt: Statement) -> TableInfo:
        # 提取表名
        tables = super()._find_table(stmt, {'EXISTS', 'TABLE'})
        return TableInfo(tables)


from typing import Any

from mag_tools.bean.db_page import DbPage
from mag_tools.enums.base_enum import BaseEnum
from mag_tools.enums.symbol import Symbol
from mag_tools.utils.common.string_utils import StringUtils

from mag_db.bean.column import Column
from mag_db.builder.sql_builder import SqlBuilder
from mag_db.enums.operator_type import OperatorType
from mag_db.enums.relation import Relation


class Where:
    """
    WHERE语句操作类

    @description: 用于构建WHERE语句。
    @author <a href="xl.cao@hotmail.com">xlcao</a>
    @version v1.1
    Copyright (c) 2016 by Xiaolong Cao. All rights reserved.
    @date: 2017/8/12
    """
    def __init__(self, sql: str = None):
        """
        构造方法
        """
        self.__builder = SqlBuilder()
        self.__fields = []
        self.__page = None

        if sql:
            if sql.upper().startswith('WHERE'):
                sql = StringUtils.pick_tail(sql, ' ')

            self.__builder.append_string(sql)

    @property
    def fields(self):
        return self.__fields

    @property
    def is_empty(self):
        """
        判定是否为空

        :return: 是否为空
        """
        return self.__builder.is_empty

    def get_page(self):
        return self.__page

    @property
    def is_page_query(self):
        return self.__page is not None

    def and_(self):
        """
        添加“AND”

        :return: Where
        """
        return self.append_type(OperatorType.AND)

    def or_(self):
        """
        添加“OR”

        :return: Where
        """
        return self.append_type(OperatorType.OR)

    def column(self, column_name: str, relation: Relation, value: Any, ignore_null_value: bool = True):
        """
        添加列参数

        :param column_name: 列名
        :param relation: 列名与参数的关系类型，大于、小于、等于、不小于、不大于、不等于、LIKE、NOT LIKE等
        :param value: 列值
        :param ignore_null_value: 是否忽略空值
        :return: Where
        """
        if isinstance(relation, str):
            relation = Relation.of_code(relation)

        if (column_name and
                (not ignore_null_value or value is not None) and
                relation in [Relation.MORE_THAN, Relation.MORE_THAN_EQUAL, Relation.LESS_THAN, Relation.LESS_THAN_EQUAL, Relation.EQUAL, Relation.NOT_EQUAL, Relation.LIKE, Relation.NOT_LIKE]):
            if self.__builder.is_empty:
                self.__builder.append_operator(OperatorType.WHERE)
            has_table_name = "." in column_name
            self.__builder.append_column(Column(name=column_name), has_table_name, False).append_relation(relation).append_symbol(Symbol.PLACE_HOLDER)
            self.add_field(value)
        else:
            if self.__builder.is_at_end(OperatorType.AND.code):
                self.__builder.remove_last_keyword(OperatorType.AND.code)
            elif self.__builder.is_at_end(OperatorType.OR.code):
                self.__builder.remove_last_keyword(OperatorType.OR.code)
        return self

    def column_value(self, column_name: str, value: Any, ignore_null_value: bool = True):
        """
        添加列参数

        :param column_name: 列名
        :param value: 列值
        :param ignore_null_value: 是否忽略空值
        :return: Where
        """
        if column_name and (not ignore_null_value or value is not None):
            if self.__builder.is_empty:
                self.append_type(OperatorType.WHERE)
            self.column(column_name, Relation.EQUAL, value)
        else:
            if self.__builder.is_at_end(OperatorType.AND.code):
                self.__builder.remove_last_keyword(OperatorType.AND.code)
            elif self.__builder.is_at_end(OperatorType.OR.code):
                self.__builder.remove_last_keyword(OperatorType.OR.code)
        return self

    def equal_column(self, column1: str, column2: str):
        """
        添加指定列等于另外一列

        :param column1: 列名
        :param column2: 列名
        :return: Where
        """
        if column1 and column2:
            if self.__builder.is_empty:
                self.append_type(OperatorType.WHERE)
            self.__builder.append_column(Column(name=column1), True, False).append_relation(Relation.EQUAL).append_column(Column(name=column2), True, False)
        else:
            if self.__builder.is_at_end(OperatorType.AND.code):
                self.__builder.remove_last_keyword(OperatorType.AND.code)
            elif self.__builder.is_at_end(OperatorType.OR.code):
                self.__builder.remove_last_keyword(OperatorType.OR.code)
        return self

    def in_select_str(self, column: str, select_str: str):
        """
        添加“IN”条件
        WHERE ... AND/OR column IN (SELECT语句)

        :param column: 列名
        :param select_str: IN的查询条件，SELECT * FROM WHERE *
        :return: Where
        """
        if column and select_str:
            if self.__builder.is_empty:
                self.append_type(OperatorType.WHERE.code)
            self.__builder.append_column(Column(name=column), True, False).append_operator(OperatorType.IN)
            self.append_paren(select_str)
        else:
            if self.__builder.is_at_end(OperatorType.AND.code):
                self.__builder.remove_last_keyword(OperatorType.AND.code)
            elif self.__builder.is_at_end(OperatorType.OR.code):
                self.__builder.remove_last_keyword(OperatorType.OR.code)
        return self

    def in_select(self, column: str, select):
        """
        添加“IN”条件
        WHERE ... AND/OR column IN (SELECT语句)

        :param column: 列名
        :param select: IN的查询条件，SELECT * FROM WHERE *
        :return: Where
        """
        if column and select:
            if self.__builder.is_empty:
                self.append_type(OperatorType.WHERE.code)
            self.__builder.append_column(Column(name=column), True, False).append_operator(OperatorType.IN)
            self.append_paren(str(select))
            if select.get_where():
                self.__fields.extend(select.get_where().__fields)
        else:
            if self.__builder.is_at_end(OperatorType.AND.code):
                self.__builder.remove_last_keyword(OperatorType.AND.code)
            elif self.__builder.is_at_end(OperatorType.OR.code):
                self.__builder.remove_last_keyword(OperatorType.OR.code)
        return self

    def in_list(self, column: str, params: list[Any]):
        """
        添加“IN”条件
        WHERE... column IN (str1, str2...)

        :param column: 列名
        :param params: IN的参数列表
        :return: Where
        """
        if column and params:
            if self.__builder.is_empty:
                self.append_type(OperatorType.WHERE.code)
            self.__builder.append_column(Column(name=column), True, False).append_operator(OperatorType.IN)
            param_strs = [f"'{param}'" if isinstance(param, str) else str(param) for param in params]
            in_str = ", ".join(param_strs)
            self.append_paren(in_str)
        else:
            if self.__builder.is_at_end(OperatorType.AND.code):
                self.__builder.remove_last_keyword(OperatorType.AND.code)
            elif self.__builder.is_at_end(OperatorType.OR.code):
                self.__builder.remove_last_keyword(OperatorType.OR.code)
        return self

    def not_in_list(self, column: str, params: list[Any]):
        """
        添加“NOT IN”条件
        WHERE... column NOT IN (str1, str2...)

        :param column: 列名
        :param params: NOT IN的参数列表
        :return: Where
        """
        if column and params:
            if self.__builder.is_empty:
                self.append_type(OperatorType.WHERE.code)
            self.__builder.append_column(Column(name=column), True, False).append_operator(OperatorType.NOT_IN)
            param_strs = [f"'{param}'" if isinstance(param, str) else str(param) for param in params]
            not_in_str = ", ".join(param_strs)
            self.append_paren(not_in_str)
        else:
            if self.__builder.is_at_end(OperatorType.AND.code):
                self.__builder.remove_last_keyword(OperatorType.AND.code)
            elif self.__builder.is_at_end(OperatorType.OR.code):
                self.__builder.remove_last_keyword(OperatorType.OR.code)
        return self

    def in_place_holder(self, column_name: str, param_count: int):
        """
        添加“IN”条件
        WHERE... column IN (?, ?...)

        :param column_name: 列名
        :param param_count: IN的参数个数
        :return: Where
        """
        if column_name and param_count > 0:
            if self.__builder.is_empty:
                self.append_type(OperatorType.WHERE.code)
            self.__builder.append_column(Column(name=column_name), True, False).append_operator(OperatorType.IN)
            self.__builder.append_symbol_with_paren(Symbol.PLACE_HOLDER, param_count)
        else:
            if self.__builder.is_at_end(OperatorType.AND.code):
                self.__builder.remove_last_keyword(OperatorType.AND.code)
            elif self.__builder.is_at_end(OperatorType.OR.code):
                self.__builder.remove_last_keyword(OperatorType.OR.code)
        return self

    def in_fields(self, column: str, fields: list[Any]):
        """
        添加“IN”条件
        WHERE... column IN (?, ?...)

        :param column: 列名
        :param fields: IN的参数列表
        :return: Where
        """
        self.in_place_holder(column, len(fields))
        self.add_fields(fields)
        return self

    def not_in(self, column: str, param_count: int):
        """
        添加“NOT IN”条件
        WHERE... column NOT IN (?, ?...)

        :param column: 列名
        :param param_count: NOT IN的参数个数
        :return: Where
        """
        if column and param_count > 0:
            if self.__builder.is_empty:
                self.append_type(OperatorType.WHERE.code)
            self.__builder.append_column(Column(name=column), True, False).append_operator(OperatorType.NOT_IN)
            self.__builder.append_symbol_with_paren(Symbol.PLACE_HOLDER, param_count)
        else:
            if self.__builder.is_at_end(OperatorType.AND.code):
                self.__builder.remove_last_keyword(OperatorType.AND.code)
            elif self.__builder.is_at_end(OperatorType.OR.code):
                self.__builder.remove_last_keyword(OperatorType.OR.code)
        return self

    def not_in_fields(self, column: str, fields: list[Any]):
        """
        添加“NOT IN”条件
        WHERE... column NOT IN (?, ?...)

        :param column: 列名
        :param fields: NOT IN的参数列表
        :return: Where
        """
        self.not_in(column, len(fields))
        self.add_fields(fields)
        return self

    def not_in_select_str(self, column: str, select_str: str):
        """
        添加“NOT IN”条件
        WHERE... column NOT IN (SELECT语句)

        :param column: 列名
        :param select_str: NOT IN的查询条件，SELECT * FROM WHERE *
        :return: Where
        """
        if column and select_str:
            if self.__builder.is_empty:
                self.append_type(OperatorType.WHERE.code)
            self.__builder.append_column(Column(name=column), True, False).append_operator(OperatorType.NOT_IN.code)
            self.append_paren(select_str)
        else:
            if self.__builder.is_at_end(OperatorType.AND.code):
                self.__builder.remove_last_keyword(OperatorType.AND.code)
            elif self.__builder.is_at_end(OperatorType.OR.code):
                self.__builder.remove_last_keyword(OperatorType.OR.code)
        return self

    def not_in_select(self, column: str, select):
        """
        添加“NOT IN”条件
        WHERE... column NOT IN (SELECT语句)

        :param column: 列名
        :param select: NOT IN的查询条件，SELECT * FROM WHERE *
        :return: Where
        """
        if column and select:
            if self.__builder.is_empty:
                self.append_type(OperatorType.WHERE.code)
            self.__builder.append_column(Column(name=column), True, False).append_operator(OperatorType.NOT_IN)
            self.append_paren(str(select))
            if select.where:
                self.__fields.extend(select.where.__fields)
        else:
            if self.__builder.is_at_end(OperatorType.AND.code):
                self.__builder.remove_last_keyword(OperatorType.AND.code)
            elif self.__builder.is_at_end(OperatorType.OR.code):
                self.__builder.remove_last_keyword(OperatorType.OR.code)
        return self

    def order(self, ascend: bool, *column_names: str):
        """
        添加“ORDER”条件
        ORDER (col1, col2)

        :param ascend: 是否升序
        :param column_names: 列名列表
        :return: Where
        """
        if column_names:
            has_table_name = "." in column_names[0]
            self.__builder.append_operator(OperatorType.ORDER_BY)

            columns = [Column(name[:-1] if name.endswith('_') else name) for name in column_names]
            self.__builder.append_without_paren(columns, has_table_name)
            if not ascend:
                self.__builder.append_operator(OperatorType.DESC)
        return self

    def like(self, column: str, value: str):
        """
        添加LIKE条件

        :param column: 列名
        :param value: 列值
        :return: Where
        """
        if column and value:
            if not value.startswith("%") and not value.endswith("%"):
                value = f"%{value}%"
            self.column(column, Relation.LIKE, value)
        else:
            if self.__builder.is_at_end(OperatorType.AND.code):
                self.__builder.remove_last_keyword(OperatorType.AND.code)
            elif self.__builder.is_at_end(OperatorType.OR.code):
                self.__builder.remove_last_keyword(OperatorType.OR.code)
        return self

    def sub_query(self, relation: str, column: str, sub_query: str):
        """
        添加子查询

        :param relation: 列名与参数的关系类型，大于、小于、等于、不小于、不大于、不等于、LIKE、NOT LIKE等
        :param column: 列名
        :param sub_query: 查询子语句
        :return: Where
        """
        if sub_query and relation in [Relation.MORE_THAN, Relation.MORE_THAN_EQUAL, Relation.LESS_THAN, Relation.LESS_THAN_EQUAL, Relation.EQUAL, Relation.NOT_EQUAL, Relation.LIKE, Relation.NOT_LIKE]:
            if self.__builder.is_empty:
                self.append_type(OperatorType.WHERE)
            self.__builder.append_column(Column(name=column), True, False).append_relation(relation)
            self.append_paren(sub_query)
        return self

    def sub_where(self, sub_where):
        """
        添加子语句

        :param sub_where: 子条件语句，不包含WHERE
        :return: Where
        """
        if sub_where:
            if self.__builder.is_empty:
                self.append_type(OperatorType.WHERE)
            if sub_where.__builder.sql.startswith(OperatorType.WHERE.code):
                sub_where.__builder.sql = sub_where.__builder.sql[5:]
            self.__builder.append_symbol(Symbol.OPEN_PAREN)
            self.__builder.append_string(sub_where.__builder.sql)
            self.__builder.append_symbol(Symbol.CLOSE_PAREN)
            self.__fields.extend(sub_where.__fields)
        return self

    def limit(self, offset: int = None, row_count: int = None):
        """
        添加LIMIT条件

        :param offset: 记录行数偏移量
        :param row_count: 最大记录数
        :return: Where
        """
        if row_count is not None:
            self.__builder.append_operator(OperatorType.LIMIT)
            if offset is not None:
                self.__builder.append_long(offset).append_symbol(Symbol.COMMA)
            self.__builder.append_long(row_count)
        return self

    def group_by(self, sql_group: str):
        """
        添加GROUP BY条件

        :param sql_group: Group语句
        :return: Where
        """
        if sql_group:
            self.__builder.append_operator(OperatorType.GROUP_BY).append_string(sql_group)
        return self

    def for_update(self):
        """
        添加FOR UPDATE条件

        :return: Where
        """
        self.__builder.append_operator(OperatorType.FOR_UPDATE)
        return self

    def __str__(self):
        """
        流化为SQL语句

        :return: SQL语句
        """
        sql = self.__builder.__str__().strip()
        if not sql.startswith(OperatorType.WHERE.code) and not any(sql.startswith(keyword) for keyword in [OperatorType.GROUP.code, OperatorType.ORDER.code, OperatorType.LIMIT.code]):
            sql = f"{OperatorType.WHERE.code} {sql}"
        return sql if len(sql) >= 8 else ""

    def append_paren(self, *sql_strs: str):
        """
        添加括号包含起来的字符串,如：(语句1，语句2...)

        :param sql_strs: SQL语句
        """
        if sql_strs:
            self.__builder.append_symbol(Symbol.OPEN_PAREN)
            for sql_str in sql_strs:
                if sql_str:
                    self.__builder.append_string(sql_str)
            self.__builder.append_symbol(Symbol.CLOSE_PAREN)

    def set_params(self, params: list[str]):
        """
        将条件语句中的多个问号替换为字符串参数

        :param params: 参数列表
        :return: Where
        """
        sql = str(self)
        for param in params:
            sql = sql.replace(Symbol.PLACE_HOLDER.code, f"'{param}'", 1)
        return Where(sql)

    def has_fields(self):
        """
        判定是否包含条件参数的值

        :return: 是否包含条件参数的值
        """
        return bool(self.__fields)

    def page(self, page):
        """
        设置分页查询参数

        :param page: 页面信息，包括：页面索引，从1开始；页面大小
        :return: Where
        """
        self.__page = page or DbPage(1, 10)
        if self.__page.order_column:
            self.order(self.__page.is_ascend, self.__page.order_column)
        if self.__builder.is_at_end("AND"):
            self.__builder.remove_last_keyword("AND")
        elif self.__builder.is_at_end("OR"):
            self.__builder.remove_last_keyword("OR")
        return self

    def add_fields(self, fields: list[Any]):
        """
        添加条件参数的值（替代问号）

        :param fields: 条件参数的值
        """
        if fields:
            for field in fields:
                self.add_field(field)

    def append_type(self, operator_type: OperatorType):
        """
        添加“AND”、“OR”、“IN”、“NOT IN”

        :param operator_type: 操作符类型
        :return: Where
        """
        if isinstance(operator_type, str):
            operator_type = OperatorType.of_code(operator_type)

        last = StringUtils.last_word(self.__builder.__str__().strip()).upper()
        if self.__builder.is_empty and operator_type in [OperatorType.AND, OperatorType.OR]:
            return self
        if last not in [OperatorType.WHERE, OperatorType.AND, OperatorType.OR, OperatorType.IN, OperatorType.NOT_IN, OperatorType.GROUP_BY, OperatorType.ORDER_BY]:
            self.__builder.append_operator(operator_type)
        return self

    def add_field(self, value: Any):
        """
        添加条件参数值，对枚举添加其value值

        :param value: 条件参数值
        """
        self.__fields.append(value.code if isinstance(value, BaseEnum) else value)


from dataclasses import dataclass, field
from typing import Optional, Type

from mag_db.data.column_names_mapping import ColumnNamesMapping

@dataclass
class DbOutput:
    """
    数据库输出信息类

    @description: 提供基础的分页操作
    @version: 2.5
    @date: 2015
    """
    result_class: Optional[Type] = None
    column_name_mapping: ColumnNamesMapping = field(default_factory=ColumnNamesMapping)
    is_multi_table: bool = False
    column_names: list[str] = field(default_factory=list)

    @classmethod
    def from_class(cls, bean_class: Optional[Type], col_names: list[str], col_name_map: Optional[dict[str, str]] = None):
        output = cls()
        output.column_names = col_names

        if bean_class:
            mapping_from_bean = ColumnNamesMapping.get_by_class(bean_class, col_names, col_name_map)
            output.column_name_mapping = mapping_from_bean
            output.result_class = bean_class

        return output

    def get_target_name(self, column_name):
        target_name = self.column_name_mapping.get_target_name(column_name) if self.column_name_mapping else column_name
        return target_name if target_name else column_name


from dataclasses import dataclass, field
from typing import Optional

@dataclass
class PrimaryKey:
    """
    描述一个主键约束
    """
    column_names: list[str] = field(default_factory=list, metadata={'description': '主键列名列表'})
    key_seq: list[int] = field(default_factory=list, metadata={'description': '主键列的顺序列表'})
    pk_name: Optional[str] = field(default=None, metadata={'description': '主键的名称'})
    increment: Optional[int] = field(default=None, metadata={'description': '自增列的增量值'})

    @property
    def default_column(self) -> str:
        return self.column_names[0] if self.column_names and len(self.column_names) == 1 else None

    def __str__(self):
        """
        返回主键约束的字符串表示

        返回:
        str: 主键约束的字符串表示
        """
        parts = [f"PRIMARY KEY ({', '.join(self.column_names)})"]
        if self.key_seq and len(self.key_seq) > 0:
            parts.append(f"KEY_SEQ ({', '.join(map(str, self.key_seq))})")
        if self.pk_name:
            parts.append(f"PK_NAME {self.pk_name}")
        if self.increment:
            parts.append(f"INCREMENT {self.increment}")
        return " ".join(parts)


from dataclasses import dataclass, field
from typing import Optional, Any, TypeVar


from mag_db.handler.type_handler import TypeHandler

T = TypeVar('T')

@dataclass
class DbParameter:
    """
    一条记录的参数及类型的列表
    """
    value: Optional[Any] = field(default=None, metadata={'description': '参数值'})
    type_handler: TypeHandler = field(default=None, metadata={'description': '参数类型'})
    
@dataclass
class RowParameter:
    """
    单条记录的参数信息
    """
    __parameters: list[DbParameter] = field(default_factory=list, metadata={'description': '一条记录的参数清单'})

    def append(self, parameter: Any, type_handler: TypeHandler) -> None:
        parameter = type_handler.set_parameter(parameter)
        self.__parameters.append(DbParameter(parameter, type_handler))
        
    @property
    def values(self) -> list[Any]:
        return [parameter.value for parameter in self.__parameters]
    
    @property
    def type_handlers(self) -> list[TypeHandler]:
        return [parameter.type_handler for parameter in self.__parameters]

@dataclass
class ParameterCache:
    row_parameters: list[RowParameter] = field(default_factory=list, metadata={'description': '多条记录的参数清单'})
    sum_of_params: int = field(default=0, metadata={'description': '记录数'})
    field_of_where: list[Any] = field(default_factory=list, metadata={'description': '条件语句中的被占位符替代的字段数值'})

    def set_parameter(self, parameter: Any, type_handler: TypeHandler, row_index: int = 0) -> None:
        if row_index < len(self.row_parameters):
            row_parameter = self.row_parameters[row_index]
        else:
            row_parameter = RowParameter()
            self.row_parameters.append(row_parameter)

        row_parameter.append(parameter, type_handler)

        self.sum_of_params += 1

    @property
    def values(self) -> tuple[tuple[Any, ...], ...]:
        rows = [tuple(row.values) for row in self.row_parameters]
        return tuple(rows)
    
    def get_values(self)-> tuple[tuple[Any, ...], ...]:
        all_values: list[tuple[Any, ...]] = []

        for row_parameter in self.row_parameters:
            parameters_of_fow = list(row_parameter.values)
            if self.field_of_where:
                parameters_of_fow.extend(self.field_of_where)

            all_values.append(tuple(parameters_of_fow))

        if len(self.row_parameters) == 0 and len(self.field_of_where) > 0:
            all_values.append(tuple(self.field_of_where))

        return tuple(all_values)
    
    def type_handlers(self, row_index: int = 0)->list[TypeHandler]:
        return self.row_parameters[row_index].type_handlers if row_index < len(self.row_parameters) else []


from dataclasses import dataclass, field
from datetime import date, datetime, time
from enum import Enum
from typing import Any, Optional, TypeVar

from mag_tools.enums.base_enum import BaseEnum
from mag_tools.utils.common.class_utils import ClassUtils
from mag_tools.utils.common.string_utils import StringUtils

from mag_db.data.parameter_cache import ParameterCache
from mag_db.data.column_names_mapping import ColumnNamesMapping
from mag_db.handler.type_constant import TypeConstant
from mag_db.handler.type_handler import TypeHandler
from mag_db.utils.column_utils import ColumnUtils
from mag_db.utils.dao_utils import DaoUtils

T = TypeVar('T')
K = TypeVar('K')
V = TypeVar('V')

@dataclass
class RowParameter:
    """
    一条记录的参数及类型的列表
    """
    parameters: list[Any] = field(default_factory=list, metadata={'description': '数据库记录的参数清单'})
    type_handlers: list[TypeHandler] = field(default_factory=list, metadata={'description': '记录的参数类型列表'})

    def append(self, parameter: Any, type_handler: TypeHandler) -> None:
        self.parameters.append(parameter)
        self.type_handlers.append(type_handler)

@dataclass
class IndexParameters:
    __parameter_cache: ParameterCache = field(default_factory=ParameterCache, metadata={'description': '多条记录的参数清单'})

    def set_value(self, parameter: Any, type_: type = None) -> None:
        type_constant = TypeConstant.STRING
        if type_ is bytes or isinstance(parameter, bytes):
            type_constant = TypeConstant.BYTES
        elif type_ is bool or isinstance(parameter, bool):
            type_constant = TypeConstant.BOOL
        elif type_ is int or isinstance(parameter, int):
            type_constant = TypeConstant.INT
        elif type_ is float or isinstance(parameter, float):
            type_constant = TypeConstant.FLOAT
        elif type_ is date or isinstance(parameter, date):
            type_constant = TypeConstant.DATE
        elif type_ == datetime or isinstance(parameter, datetime):
            type_constant = TypeConstant.DATETIME
        elif type_ is time or isinstance(parameter, time):
            type_constant = TypeConstant.TIME
        elif type_ is tuple or isinstance(parameter, tuple):
            type_constant = TypeConstant.TUPLE

        self.__parameter_cache.set_parameter(parameter, type_constant)

    def set_beans(self, param_beans: list[T], column_names: list[str],
                  column_name_map: Optional[dict[str, str]] = None) -> None:
        if param_beans:
            column_names_mapping = ColumnNamesMapping.get_by_class(type(param_beans[0]), column_names, column_name_map)
            field_names = ColumnUtils.to_field_names(column_names, column_names_mapping)
            for row_index, param_bean in enumerate(param_beans):
                self.__put_bean(param_bean, field_names, row_index)

    def set_field_maps(self, field_maps: list[dict[K, V]], column_names: list[str]) -> None:
        if field_maps:
            column_names_mapping = ColumnNamesMapping.get_by_field_map(field_maps[0], column_names)
            field_names = ColumnUtils.to_field_names(column_names, column_names_mapping)
            for row_index, field_map in enumerate(field_maps):
                self.__put_map(field_map, field_names, row_index)

    def set_fields_of_where(self, params: list[T]) -> None:
        self.__parameter_cache.field_of_where = []
        for param in params:
            if isinstance(param, Enum):
                self.__parameter_cache.field_of_where.append(param.code if isinstance(param, BaseEnum) else param.name)
            else:
                self.__parameter_cache.field_of_where.append(param)

    @property
    def row_count(self) -> int:
        row_count = len(self.__parameter_cache.row_parameters)
        return row_count if row_count > 0 else len(self.__parameter_cache.field_of_where)

    @property
    def sum_of_params(self) -> int:
        return self.__parameter_cache.sum_of_params

    @property
    def field_num(self):
        return len(self.__parameter_cache.field_of_where)

    @property
    def parameters(self)->tuple[tuple[Any,...],...]:
        return self.__parameter_cache.values

    def type_handlers(self, row_index: int = 0)->list[TypeHandler]:
        return self.__parameter_cache.type_handlers(row_index)

    def get_values(self)-> tuple[tuple[Any, ...], ...]:
        return self.__parameter_cache.get_values()

    def clear(self):
        self.__parameter_cache.row_parameters.clear()
        self.__parameter_cache.sum_of_params = 0

    def is_empty(self) -> bool:
        return len(self.__parameter_cache.row_parameters) == 0 and len(self.__parameter_cache.field_of_where) == 0

    # def __set_parameter(self, parameter: Any, type_handler: TypeHandler, row_index: int = 0) -> None:
    #     if row_index < len(self.__row_parameters):
    #         row_parameter = self.__row_parameters[row_index]
    #     else:
    #         row_parameter = RowParameter()
    #         self.__row_parameters.append(row_parameter)
    #
    #     row_parameter.append(parameter, type_handler)
    #
    #     self.__sum_of_params += 1

    def __put_bean(self, param_bean: T, column_names: list[str], row_index: int) -> None:
        for column_name in column_names:
            bean_field_name = StringUtils.hump2underline(column_name)

            try:
                bean_field_value = getattr(param_bean, bean_field_name, None)
                bean_field_type = ClassUtils.get_field_type_of_bean(param_bean, bean_field_name)
                bean_field_type = ClassUtils.get_origin_type(bean_field_type)

                if isinstance(bean_field_value, Enum):
                    bean_field_value = bean_field_value.code if hasattr(bean_field_value, 'code') else bean_field_value.name

                type_handler = DaoUtils.get_type_handler(bean_field_type)
                self.__parameter_cache.set_parameter(bean_field_value, type_handler, row_index)
            except AttributeError:
                pass  # 该列名在bean中不存在，跳过

    def __put_map(self, field_set: dict[str, V], column_names: list[str],  row_index: int) -> None:
        for column_name in column_names:
            field_value = field_set.get(column_name)
            field_type = type(field_value)
            if field_type is None:
                field_type = str

            type_handler = DaoUtils.get_type_handler(field_type)
            self.__parameter_cache.set_parameter(field_value, type_handler, row_index)
